local fontPath = "Interface\\AddOns\\DCD\\font.ttf";
local pixel	= "Interface\\AddOns\\DCD\\pixel";

-- Public Frames for the mod
local DCD = nil;
local Anchor = nil;

-- Player Information
local PlayerClass = select(2, UnitClass("Player"));
local PlayerRace = select(2, UnitRace("Player"));

-- Creates a list of spells available for the players class, race, pet and items
local availableSpells = {};
local availableRaceSpells = {};
local availableItems = {};
local availablePetSpells = {};
local spellFrames = {};
local itemFrames = {};



-- Returns the how long a spells cooldown is
function RemainingCD(frame)
		
	-- Checks if its an item or a spell
	if frame.IsItem == false then
		frame.Start, frame.Duration, frame.Enabled = GetSpellCooldown(frame.Id);
	elseif frame.IsItem == true then
		frame.Start, frame.Duration, frame.Enabled = GetItemCooldown(frame.Id);
	end
	
	remaining = frame.Start + frame.Duration - GetTime();

	if remaining <= frame.Duration and remaining > 0 and frame.Enabled == 1 and frame.Duration > 1.5 then 
		result = remaining
	else 
		result = 0;
	end
	
	return result;
	
end

function ConvertToMinutes(count)
	
	minutes = count / 60;
	seconds = math.floor(((minutes - math.floor(minutes))* 60)+ 0.5);
	
	-- Fix so it doesnt show x:60 minutes. Need to do this since the calculations are not very precise.
	if seconds == 60 then
		seconds = 59;
	end
	
	if seconds < 10 then
		seconds = "0"..seconds;
	end
	
	return math.floor(minutes), seconds
	
end


-- Update the information about a cooldown
function Update(self)

	self.PrevCount = self.Count;
	self.Count = RemainingCD(self);
	self.bar:SetValue(self.Count / self.Duration);
	
	-- Convert to minutes if its more than 60 seconds remaining.
	if self.Count > 60 then
		m, s = ConvertToMinutes(self.Count);
		self.text:SetText(m.." : "..s);
	else
		self.text:SetText(math.floor(self.Count));
	end
	
	if self.Count <= 2 then
		self.alert:SetTexture(unpack(Color["BackdropFinished"]));
		self.alert:Show();
	else
		self.alert:Hide();
	end
	
	-- Hides the spellFrame if the cooldown is at 0
	if self.Count == 0 then
		self:Hide();
		self.alert:Hide();
		
		-- Removes the spell from the number of active spells count.
		if self.IsCounted == true then
			if self.PrevCount >= UpperRowTime then
				--print("Removing from High");
				Anchor.VisibleHighTimeItems = Anchor.VisibleHighTimeItems - 1;
			elseif self.PrevCount < UpperRowTime then
				--print("Removing from Low");
				Anchor.VisibleLowTimeItems = Anchor.VisibleLowTimeItems - 1;
			end
			self.IsCounted = false;
			-- Checks if a spell/item is just done with its cooldown
			-- We do this so the positions of the icons are instantly set when a spell is off cooldown.
			if self.PrevCount ~= self.Count then
				UpdateAll();
			end
		end
	-- Show the spellFrame if the cooldown is over 0
	elseif self.Count > 0 then
		self:Show();
		-- Adds the spell to the number of active spells count.
		if self.IsCounted == false then
			if self.Count >= UpperRowTime then
				--print("Adding to High");
				Anchor.VisibleHighTimeItems = Anchor.VisibleHighTimeItems + 1;
				self:SetAlpha(0.5);
			else
				--print("Adding  to Low");
				Anchor.VisibleLowTimeItems = Anchor.VisibleLowTimeItems + 1;
				self:SetAlpha(1);
			end
			self.IsCounted = true;
			
			-- Checks if a spell/item is just went on cooldown
			-- We do this so the positions of the icons are instantly set when a spell initialize a cooldown.
			if self.PrevCount ~= self.Count then
				UpdateAll();
			end
		end
	end
	
	-- Update the low/high count if the spell has a remaining cooldown lower than "UpperRowTime"
	if self.PrevCount >= UpperRowTime and self.Count < UpperRowTime then
		--print("Shift: "..self.Id.. " from High to Low");
		Anchor.VisibleHighTimeItems = Anchor.VisibleHighTimeItems - 1;
		Anchor.VisibleLowTimeItems = Anchor.VisibleLowTimeItems + 1;
		
		self:SetAlpha(1);
		-- Trigger UpdateAll so the positions of the spells are updated correctly when changed from a HighTime spell to a LowTime spell.
		UpdateAll();
	end
end

-- Common values when creating a new frame.
function CommonFrame()
	
	frame = CreateFrame("Frame", nil, UIParent);
	height = Anchor:GetHeight();
	width = Anchor:GetWidth();
	frame.IsCounted = false;
	
	frame:SetHeight(height);
	frame:SetWidth(width);
	frame:SetPoint("CENTER", 0, 0);
	
	frame.Count = 0;
	frame.PrevCount = 0;
	frame.Start = 0;
	frame.Duration = 0;
	frame.Enabled = 0;
		
	frame:SetBackdrop( {
		bgFile = pixel;
		edgeFile = pixel,
		tile = false,
		tileSize = 0,
		edgeSize = 1,
		insets = { left = 1, right = 1, top = 1, bottom = 1 },
	});
	frame:SetBackdropColor(unpack(Color["Backdrop"]));
	frame:SetBackdropBorderColor(unpack(Color["BackdropBorder"]));
		
	-- SpellImage
	frame.texture = frame:CreateTexture();
	frame.texture:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 2, 2);
	frame.texture:SetHeight(height - 8);
	frame.texture:SetWidth(width - 4);
	frame.texture:SetTexCoord(0.08,0.92,0.08,0.92)
	
	-- Countdown bar
	frame.bar = CreateFrame("StatusBar", nil, frame, "TextStatusBar");
	frame.bar:SetStatusBarTexture(pixel);
	frame.bar:GetStatusBarTexture():SetVertexColor(0, 0, 0);
	frame.bar:SetPoint("TOPLEFT", frame, "TOPLEFT", 2, -2);
	frame.bar:SetHeight(3);
	frame.bar:SetWidth(width - 4);
	frame.bar:SetMinMaxValues(0, 1);
	
	-- Alert Overlay
	frame.alert = frame:CreateTexture(nil, "OVERLAY")
	frame.alert:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 2, 2);
	frame.alert:SetHeight(height - 8);
	frame.alert:SetWidth(width - 4);
	frame.alert:SetTexture(0, 0, 0, 0);
	frame.alert:SetBlendMode("MOD")
	
	-- Spell Timer Text
	frame.text = frame:CreateFontString(nil, "OVERLAY");
	frame.text:SetFont(fontPath, 12, "OUTLINE");
	frame.text:SetShadowColor(0,0,0,1)
	frame.text:SetText("");
	frame.text:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 2, 2);
		
	return frame;
	
end

-- Creates a new Cooldown spell
function NewSpellFrame(Spell)
		
	frame = CommonFrame();
	frame.Id = Spell;
	frame.IsItem = false;
		
	-- Spell Image
	frame.texture:SetTexture(GetSpellTexture(Spell));

	frame:Hide();
	
	frame:SetScript("OnUpdate", Update);
	
	return frame;
end 

-- Creates a new Cooldown item
function NewItemFrame(Item)
		
	frame = CommonFrame();
	frame.Id = Item;
	frame.IsItem = true;

	-- Item Image
	local texture = select(10, GetItemInfo(Item));
	frame.texture:SetTexture(texture);
	
	frame:Hide();
	
	frame:SetScript("OnUpdate", Update);
	
	return frame;
end 

-- Creates frames for each of the spells available for the player
function InitializeItems()
	
	--print("Initializing");
	for i,v in ipairs(availableSpells) do
		if GetSpellInfo(GetSpellInfo(v)) then
			spellFrames[v] = spellFrames[v] or NewSpellFrame(v);
			spellFrames[v].bar:GetStatusBarTexture():SetVertexColor(unpack(Color["BarSpells"]));
			--print("Init S: ", v, " - ", spellFrames[v].Name);
		end
	end
	
	for i,v in ipairs(availableRaceSpells) do
		if GetSpellInfo(GetSpellInfo(v)) then
			spellFrames[v] = spellFrames[v] or NewSpellFrame(v);
			spellFrames[v].bar:GetStatusBarTexture():SetVertexColor(unpack(Color["BarRaceSpells"]));
			--print("Init S: ", v);
		end
	end
	
	for i,v in ipairs(availablePetSpells) do
		if GetSpellInfo(GetSpellInfo(v)) then
			spellFrames[v] = spellFrames[v] or NewSpellFrame(v);
			spellFrames[v].bar:GetStatusBarTexture():SetVertexColor(unpack(Color["BarPetSpells"]));
			--print("Init S: ", v);
		end
	end
	
	for i,v in ipairs(availableItems) do
		if GetItemInfo(v) then
			itemFrames[v] = itemFrames[v] or NewItemFrame(v);
			itemFrames[v].bar:GetStatusBarTexture():SetVertexColor(unpack(Color["BarItems"]));
			--print("Init I: ", v);
		end
	end
end

-- Calculates the position of induvidual frames
function FramePosCalculation(count, width, gap)
	
	return count * (width + gap);
	
end

-- Calculates the position and offset from the Anchor.
function Position(count, visibleItems, anchorPos, frameSize, gap)
		
	if GrowthDirection == "CENTER" then
		offsetAnchor = (FramePosCalculation(visibleItems - 1, frameSize, gap) / 2);
		framePosition = FramePosCalculation(count, frameSize, gap);
	elseif GrowthDirection == "LEFT" then
		offsetAnchor = 0;
		framePosition = -FramePosCalculation(count, frameSize, gap);
	elseif GrowthDirection == "RIGHT" then
		offsetAnchor = 0;
		framePosition = FramePosCalculation(count, frameSize, gap);
	end

	return anchorPos - offsetAnchor + framePosition;
	
end


-- Updates all cooldowns available for the player when the event "SPELL_UPDATE_COOLDOWN" is triggered.
-- We do this to see if any new items are on cooldown.
function UpdateAll()
	
	local lowTimeCount = 0;
	local highTimeCount = 0;
	local point, _, _, xOff, yOff = Anchor:GetPoint();
	
	for i,item in pairs(spellFrames) do
		Update(item);
		if item.Count > 0 then
			if item.Count < UpperRowTime then
				xPos = Position(lowTimeCount, Anchor.VisibleLowTimeItems, xOff, item:GetWidth(), Gap);
				yPos = yOff;
				lowTimeCount = lowTimeCount + 1;
			elseif item.Count >= UpperRowTime then
				xPos = Position(highTimeCount, Anchor.VisibleHighTimeItems, xOff, item:GetWidth(), Gap);
				yPos = yOff - item:GetHeight() - Gap;
				highTimeCount = highTimeCount + 1;
			end
			item:SetPoint(point, xPos, yPos);
		end
	end
	
	for i, item in pairs(itemFrames) do
		Update(item);
		if item.Count > 0 then
			if item.Count < UpperRowTime then
				xPos = Position(lowTimeCount, Anchor.VisibleLowTimeItems, xOff, item:GetWidth(), Gap);
				yPos = yOff;
				lowTimeCount = lowTimeCount + 1;
			elseif item.Count >= UpperRowTime then
				xPos = Position(highTimeCount, Anchor.VisibleHighTimeItems, xOff, item:GetWidth(), Gap);
				yPos = yOff - item:GetHeight() - Gap;
				highTimeCount = highTimeCount + 1;
			end
			item:SetPoint(point, xPos, yPos);
		end
	end
	--print("LowItem: ", Anchor.VisibleLowTimeItems);
	--print("HighItem: ", Anchor.VisibleHighTimeItems);
end
	
function NewEvent(self, event, arg1)
	
	-- Create all the frames and store it when you enter an area or logges into the game
	if event == "SPELLS_CHANGED" or event == "PLAYER_ENTERING_WORLD" or event == "PLAYER_TALENT_UPDATE" then
		InitializeItems();
	end
	
	if event == "SPELL_UPDATE_COOLDOWN" or event == "SPELL_UPDATE_USABLE" or event == "ACTIONBAR_UPDATE_COOLDOWN" or event == "BAG_UPDATE_COOLDOWN" or event == "PET_BAR_UPDATE_COOLDOWN" then
		UpdateAll();
	end
	
	if event == "ADDON_LOADED" and arg1 == "DCD" then
		DCD_AnchorX = DCD_AnchorX or 0;
		DCD_AnchorY = DCD_AnchorY or 0;
		Anchor:SetPoint("CENTER", DCD_AnchorX, DCD_AnchorY);
	end
	
end

function ClearActiveItems()

	for i,item in ipairs(spellFrames) do
		item:SetScript("OnUpdate", nil);
		item:Hide();
		item.Count = 0;
	end
	for i,item in ipairs(itemFrames) do
		item:SetScript("OnUpdate", nil);
		item:Hide();
		item.Count = 0;
	end
	
end

function CreateAnchor()
	
	--print("Create Anchor!");
	Anchor = CreateFrame("Frame", nil, UIParent);
	
	Anchor.VisibleLowTimeItems = 0;
	Anchor.VisibleHighTimeItems = 0;
	
	Anchor:SetHeight(IconSize + 4); -- 4 = bar size
	Anchor:SetWidth(IconSize);
	Anchor:SetPoint("CENTER", 0, -100);
	Anchor.Texture = Anchor:CreateTexture("bgtexture", "BACKGROUND");
	Anchor.Texture:SetAllPoints(Anchor);
	Anchor.Texture:SetTexture(0, 0, 0.5, 0.50);
	
	Anchor:SetMovable(true);
	Anchor:EnableMouse(true);
	Anchor:SetScript("OnMouseUp", Anchor.StopMovingOrSizing);
	Anchor:SetScript("OnMouseDown", Anchor.StartMoving);
	
	Anchor:Hide();
	
end

function CreateAvailableSpells()

	--print("CreateAvailableSpells!");
	availableSpells = spellslist[PlayerClass];
	
	race = spellslist["RACE"];
	availableRaceSpells = race[PlayerRace];
	
	if PlayerClass == "DEATHKNIGHT" or PlayerClass == "HUNTER" or PlayerClass == "MAGE" or PlayerClass == "WARLOCK" then
		availablePetSpells = spellslist["PET"][PlayerClass];
	end
	
	availableItems = spellslist["TRINKETS"];
	
end

function CreateBase()

	DCD = CreateFrame("Frame");
	
	DCD:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	DCD:RegisterEvent("SPELL_UPDATE_USABLE");
	DCD:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
	DCD:RegisterEvent("BAG_UPDATE_COOLDOWN");
	DCD:RegisterEvent("PET_BAR_UPDATE_COOLDOWN");
	
	DCD:RegisterEvent("SPELLS_CHANGED");
	DCD:RegisterEvent("PLAYER_TALENT_UPDATE")
	DCD:RegisterEvent("PLAYER_ENTERING_WORLD");
	
	DCD:RegisterEvent("ADDON_LOADED")
	
	DCD:SetScript("OnEvent", NewEvent);
	
end

SLASH_DCD1 = "/dcd";
function SlashCmdList.DCD(msg, editbox)
	if Anchor:IsVisible() then
		DCD_AnchorX = select(4, Anchor:GetPoint());
		DCD_AnchorY = select(5, Anchor:GetPoint());
		ClearActiveItems();
		InitializeItems();
		Anchor:Hide();
	else
		Anchor:Show();
	end
	--print("Low: ", Anchor.VisibleLowTimeItems);
	--print("High: ", Anchor.VisibleHighTimeItems);
end

-----------------
-- Main Script --
-----------------
print("DCD started");
CreateAnchor();
CreateAvailableSpells();
CreateBase();
